; NAME: 
;      ADDARR
; 
; PURPOSE:
;      To shift each image in an image cube a non-integer designated number
;      of pixels and sum them or find the robust average.
; 
; CALLING SEQUENCE:
;      addarr, images, shift, imout, [help=help]
; 
; INPUTS: 
;      images- Cube of images to be shifted and summed. There must be at
;              least two images. Images must be at least 4 X 4 pixels.
;      shift- Array with x and y shift amounts.  2 x N array for N+1 images
; 
; OPTIONAL KEYWORD INPUTS: 
;      robust- Set to return robust average of the data cube, rather 
;              than the sum.
;      help- Set to nonzero scalar to have ADDARR print syntax
; 
; OUTPUT: 
;      imout- Resulting shifted and summed image.  If ADDARR is unable to 
;             calculate result, then imout=0 is returned.
; 
; EXTERNAL CALLS: 
;      ACRE, BOOST_ARRAY
; 
; COMMON BLOCKS: 
;      NONE
; 
; MODIFICATION HISTORY: 
;     - Written by Eric Volquardsen, NASA IRTF, Oct. 2002 
;     - Added ROBUST keyword, ELV, May 2004
; 
; =========================================================================
PRO addarr, images, shift, imout, robust = robust, help = help

; CHECK INPUTS 
  IF n_elements(images) LT 32 THEN help = 1 
  IF n_elements(shift) LT 2 THEN help = 1 
  IF n_elements(help) EQ 1 THEN BEGIN
       IF help NE 0 THEN BEGIN
            print, '************************************************'
            print, '***               ADDARR SYNTAX              ***'
            print, '*** > addarr, images, shift, imout,          ***'
            print, '***           robust=robust                  ***'
            print, '***                                          ***'
            print, '*** images- Cube of images to be shifted and ***'
            print, '***         summed. There must be at least   ***'
            print, '***         two images. Images must be at    ***' 
            print, '***         least 4 X 4 pixels.              ***' 
            print, '*** shift- Array with x and y shift amounts. ***'
            print, '***        2 x N array for N+1 images        ***' 
            print, '*** imout- Resulting shifted and summed      ***'
            print, '***        image. If addarr undable to       ***' 
            print, '***        calculate result, then imout=0 is ***' 
            print, '***        returned.                         ***' 
            print, '*** robust- Set to return robust average of  ***' 
            print, '***         data cube, rather than sum       ***'
            print, '************************************************'
            RETURN
       ENDIF 
  ENDIF
  IF n_elements(robust) NE 1 THEN robust = 0

; GET SIZES OF INPUTS
  szi = size(images, /dimensions)
  szs = size(shift, /dimensions)
  IF n_elements(szs) EQ 1 THEN n = 1 ELSE n = szs[1]

; CHECK THAT SIZES MATCH
  IF n_elements(szi) EQ 2 THEN BEGIN
       imout = images
       RETURN
  ENDIF
  IF n_elements(szi) NE 3 THEN BEGIN
       print, '**************************************'
       print, '***             ADDARR             ***'
       print, '*** Image has incorrect dimensions ***'
       print, '**************************************'
       print, string(7b), format='(a,$)'  
       wait, 0.3
       print, string(7b), format='(a,$)' 
       wait, 0.3 
       print, string(7b), format='(a,$)' 
       imout = 0 
       RETURN
  ENDIF
  IF (n+1) NE szi[2] THEN BEGIN
       nn = n
       n = szi[2]-1
       shnull = [0,0]
       FOR q = 1, (n-nn) DO boost_array, shift, shnull
  ENDIF

; CREATE OUTPUT IMAGE
; Initialize shifted images
  imf = images
  imout = imf[*,*,0]

; Create shifted index arrays
  FOR s = 0, (n-1) DO BEGIN
       xind = findgen(szi[0])
       yind = findgen(szi[1])
       xind = xind + shift[0,s]
       yind = yind + shift[1,s]

; Interpolate for fractional shifts to get image value
       simf = fltarr(szi[0], szi[1])
       simf[*,*] = imf[*,*,(s+1)]
       simf = interpolate(simf, xind, yind, /grid)

; Sum Images
       imout = imout + simf

  ENDFOR   ; s index

; ROBUST AVERAGE 
  IF robust NE 0 THEN BEGIN 
       imout = imout / n
       ACRE, imout, imout, 3, 3
  ENDIF 

END
