; NAME: 
;      CGMEDF  (Column Group MEDian Filter)
; 
; PURPOSE: 
;      Perform median filtering on and between groups of image
;      columns.  Designed for preprocessing of MIRSI images.
; 
; PROCEDURE: 
;      1) Divides an image into groups of columns
;      2) Finds the median of corresponding pixels in each group
;      3) Subtracts "median group" from each group
;      4) Finds median value of all pixels in each group
;      5) Subtract group median from group 
; 
; CALLING SEQUENCE: 
;      imout = cgmedf(imin, cols, [ss=ss, ext=ext, help=help])
; 
; INPUTS: 
;      imin- Input image (must be at least 20 x 20)
;      cols- Number of columns in a group (default is one column)
;            If the number of image colums is not a multiple of columns
;            then the remaining columns will not be used in determining
;            the median, but will be filtered.
; 
; OPTIONAL KEYWORD INPUTS: 
;      ss - Two element vector with starting and stoping columns
;      ext - Set if object is extended
;      help- Set to nonzero scalar value to have CGMEDF print syntax help.
; 
; OUTPUT: 
;      imout- Resulting filtered image
; 
; EXTERNAL CALLS: 
;      MEDARR
; 
; COMMON BLOCKS: 
;      NONE
; 
; MODIFICATION HISTORY: 
;      -Written by Eric Volquardsen, NASA IRTF, December 2002
;      -Modified for extended objects by ELV, NASA IRTF, November 2003
; 
; ==========================================================================
FUNCTION cgmedf, imin, cols, imout, ss=ss, ext=ext, help=help

; CHECK INPUTS
     IF n_elements(help) EQ 1 THEN BEGIN
          IF help NE 0 THEN help = 1 
     ENDIF ELSE help = 0 
     IF n_elements(imin) LT 400 THEN help = 1 
     IF n_elements(cols) NE 1 THEN cols = 1
     sinim = size(imin, /dimensions)
     IF n_elements(sinim) NE 2 THEN help = 1 
     IF cols GT (sinim[0]/2) THEN help = 1 
     IF help EQ 1 THEN BEGIN
          print, '**********************************************'
          print, '***             CGMEDF SYNTAX              ***'
          print, '*** > imout = cgmedf(imin, cols, ss=ss,    ***' 
          print, '***           ext=ext)                     ***'
          print, '***                                        ***' 
          print, '*** imin - Input image at least 20x20 pxls ***' 
          print, '***        (2-D images only)               ***'
          print, '*** cols - Number of columns in a group.   ***' 
          print, '***        (default is one column) If the  ***' 
          print, '***        number of colums is not a       ***' 
          print, '***        multiple of cols then the       ***' 
          print, '***        remaining columns will not be   ***' 
          print, '***        used in determining the median, ***' 
          print, '***        but will be filtered.           ***'
          print, '*** ss - Starting and stoping columns for  ***'
          print, '***      creating median column group. The ***'
          print, '***      reason for this keyword is to     ***' 
          print, '***      avoid using extended objects in   ***' 
          print, '***      the filtering process.            ***' 
          print, '*** ext - Set for extended objects         ***' 
          print, '*** imout- Resulting filtered image        ***'
          print, '**********************************************'
          RETURN, 0 
     ENDIF
     IF n_elements(ss) NE 2 THEN ss = [0, (sinim[0]-1)]
     IF (ss[1]-ss[0]) LT 2*cols THEN BEGIN
          print, 'INCORRECT VALUES FOR STARTING/STOPING COLUMNS'
          RETURN, 0 
     ENDIF
     div =  1.0
     IF n_elements(ext) EQ 1 THEN BEGIN
          IF ext NE 0 THEN BEGIN 
               ext = 1 
               div =  2.0
          ENDIF 
     ENDIF ELSE ext = 0 
     ext_o = -1.0

; GET COLUMN GROUPS
     ncg = fix(sinim[0]/cols)
     remain = sinim[0] - ncg*cols
     array1 = fltarr(cols, sinim[1], ncg)
     remar = imin[remain:(sinim[0]-1), *]
     FOR j = 0, (ncg-1) DO BEGIN
          array1[*,*,j] = imin[(j*cols):((j+1)*cols-1),*]
     ENDFOR
     array1a = fltarr(cols, sinim[1], fix((ss[1]-ss[0]+1)/cols))
     FOR j = (ss[0]/cols), ((ss[1]+1)/cols-1) DO BEGIN
          array1a[*,*,(j-ss[0]/cols)] = imin[(j*cols):((j+1)*cols-1),*]
     ENDFOR

; CREATE MEDIAN COLUMN GROUP
     array2 = fltarr(cols, sinim[1])
     medarr, array1a, array2

; SUBTRACT MEDIAN GROUP FROM EACH GROUP
     array3 = fltarr(sinim[0], sinim[1])
     FOR j = 0, (ncg-1) DO BEGIN 
          array3[(j*cols):((j+1)*cols-1),*] = array1[*,*,j]-array2
     ENDFOR
     IF remain NE 0 THEN BEGIN
          remar2 = remar - array2[0:(remain-1), *]
          array3[(sinim[0]-remain):(sinim[0]-1), *] = remar2
     ENDIF

; ADJUST EACH GROUP MEDIAN TO ZERO
; Do not get median for  column groups where extended objects 
; create large wings in histogram (negative kurtosis)
     imout = array3
     FOR j = 0, (ncg-1) DO BEGIN         
          kurt = kurtosis(array3[j*cols:(j+1)*cols-1]) 
          md = 0.0 
          IF ext EQ 0 THEN kurt = ext_o + 1
          IF kurt GT ext_o THEN $ 
               md = MEDIAN(imout[j*cols:(j+1)*cols-1, *], /even) 
          md = md / div  ; not sure why, but this is needed
          imout[(j*cols):((j+1)*cols-1), *] = $ 
               imout[(j*cols):((j+1)*cols-1), *] - md 
     ENDFOR
     IF remain NE 0 THEN BEGIN
          md = median(remar2, /even)
          imout[(ncg*cols):(sinim[0]-1), *] = remar2 - md
     ENDIF

; RETURN FILTERED IMAGE
     RETURN, imout

END   
