; NAME:
;      JFLUX
;
; PURPOSE:
;      To convert between magnitude and flux.  Uses UKIRT system for 
;      zero-point spectral flux density.
;
; CALLING SEQUENCE:
;      answer = jflux(mag, band, [uncer=uncer, unt=unt, inv=inv, help=help])
;
; INPUTS:
;      mag- Input value (magnitude or flux)
;      band- Filter wavelength in microns (2-22 microns)
;
; OPTIONAL KEYWORD INPUTS:
;      uncer- Uncertainty of input
;      unt- Desired flux units: 
;             0 - Janskys
;             1 - Watts per (square meter micron)
;             2 - Ergs per (second square centimeter micron)
;      inv- A nonzero integer will cause JFLUX to calculate magnitude
;           from flux
;      help- If set to nonzero scalar then JFLUX prints syntax help
;            and returns
;
; OUTPUT:
;      answer- Two element vector with flux and flux error.  If UNCER
;              keyword was not included in call then flux error is
;              listed as 0.  Gives magnitude and magnitude error if 
;              INV is set.
;
; EXTERNAL CALLS:
;      NONE
;
; COMMON BLOCKS:
;      NONE
;
; MODIFICATION HISTORY:
;      -Written by Eric Volquardsen, NASA IRTF, Jan. 2003
;
; ========================================================================
FUNCTION jflux, mag, band, unt=unt, uncer=uncer, inv=inv, help=help, answer

; CHECK INPUTS
   qt = 0
   IF n_elements(mag) NE 1 THEN qt = 1
   IF n_elements(band) NE 1 THEN qt = 1 ELSE BEGIN 
        IF band LT 2 THEN qt = 1
        IF band GT 22 THEN qt = 1
   ENDELSE 
   IF n_elements(help) EQ 1 THEN BEGIN
        IF help NE 0 THEN qt = 1 
   ENDIF
   IF qt EQ 1 THEN BEGIN
        print, '********************************************************'
        print, '***                   JFLUX SYNTAX                   ***'
        print, '*** >answer = jflux(mag, band, uncer=unc, unt=unt, $ ***'
        print, '***        inv=inv)                                  ***'
        print, '***                                                  ***'
        print, '*** mag- Input value (magnitude or flux)             ***'
        print, '*** band- Filter wavelength in microns (2-22)        ***'
        print, '*** uncer- Uncertainty of input                      ***'
        print, '*** unt- Desired flux units:                         ***'
        print, '***             0 - Janskys                          ***'
        print, '***             1 - Watts per (square meter micron)  ***'
        print, '***             2 - Ergs per (second sq cm micron)   ***'
        print, '*** inv- A nonzero integer will cause JFLUX to       ***'
        print, '***      calculate magnitude from flux               ***' 
        print, '********************************************************'
        RETURN, [0,0]
   ENDIF
   IF n_elements(uncer) NE 1 THEN uncer = 0
   IF n_elements(unt) NE 1 THEN unt = 0 
   IF unt GT 2 THEN unt = 0 
   IF unt LT 0 THEN unt = 0 
   unt = fix(unt)
   mag = float(mag)
   band = float(band)

; INITIALIZE VALUES
   IF band LT 10 THEN coef = [3113.63, (-2049.71), 580.221, $
        (-83.1612), 5.91302, (-0.165698)] $
   ELSE coef = [(-1188.99), 480.795, (-71.2934), 5.04345, $
        (-0.171985), 0.00227214]
   x = [1, band, band^2, band^3, band^4, band^5]


; SEPARATE FLUX AND MAGNITUDE PROBLEMS
   IF n_elements(inv) NE 1 THEN inv = 0 
   IF inv NE 0 THEN BEGIN

; Convert input flux to Janskys (if needed)
        CASE unt OF

             1 : mag = (3.0e-12)*mag/(band^2)

             2 : mag = (3.0e-9)*mag/(band^2)

             ELSE: 

        ENDCASE

; Calculate magnitude from flux
        answer = (-2.5)*alog10(mag/(total(coef*x)))
        junc = (-2.5)*alog10((mag+uncer)/(total(coef*x)))
        junc = abs(junc - answer)
        answer = [answer, junc]

   ENDIF ELSE BEGIN

; Calculate flux in Janskys
        answer = (total(coef*x))*(10^(mag/(-2.5)))
        junc = (total(coef*x))*(10^((mag+uncer)/(-2.5)))
        junc = abs(junc - answer)
        answer = [answer, junc]

; Calculate flux in Watts per square meter micron
        IF unt EQ 1 THEN answer = (3.0e-12)*answer/(band^2)

; Calculate flux in Ergs per second square cm micron
        IF unt EQ 2 THEN answer = (3.0e-9)*answer/(band^2)

   ENDELSE

   RETURN, answer

END
