; NAME: 
;      MIRSI_REGISTER
; 
; PURPOSE: 
;      IRTF MIRSI reduction pipeline subroutine that registers 
;      images that have been previously coadded.
; 
; CALLING SEQUENCE: 
;      mirsi_register, infiles, outfile, path, [dither=dither, 
;                      plot=plot, step=step, peak=peaks, help=help]
; 
; INPUTS: 
;      infiles - Filenames of images to register.  Files should have 
;                320x240x2 coadded image cube with MEANTIME keyword
;                in header.  If step=2 then there should be only one 
;                file in infiles.  Otherwise there should be at least
;                two files.
;      outfile - Filename where registered images should be saved
; 
; OPTIONAL INPUTS:
;      path- 2-element string vector listing paths 
;
; OPTIONAL KEYWORD INPUTS: 
;      dither - 2n element vector with x and y displacements for an 
;               n point dither pattern (in pixels)  
;      plot - Set to nonzero scalar to display images saved to
;             outfile. 
;      mshft - Maximum allowable shift in pixels (default is 10) 
;              If two-element vector is passed, the first element 
;              is the maximum allowable shift and the second element
;              is the maximum allowable difference between calculated
;              positions and estimated positions (peaks). 
;      step - Set for desired registration step to execute: 
;                   0 : Both steps (default)
;                   1 : First registration step (combines all files)
;                   2 : Second registration step (combines coad subimages
;                       (in output from first step)   
;      peaks - Estimates of peak locations for each coadded image
;              resulting in a vector of 4 * # of input images
;              ([x1_i1, y1_i1, x2_i1, y2_i1, x1_i2, y1_i2, ...])
;      lims - Two element vector of minimum and maximum allowable
;             values for removing hot pixels.
;      help- Set to nonzero scalar to print syntax. 
; 
; OUTPUT:
;      If step=1 then a 320x240x2 image cube is saved in FITS format 
;      (similar to input files).  Otherwise an 81x81 image is saved.
; 
; EXTERNAL CALLS: 
;      ADDARR, FULLWID_HALFMAX, FXADDPAR, FXPAR, FXWRITE, GETIM, 
;      IM_STATS, MCSHIFT, SXDELPAR
; 
; COMMON BLOCK: 
;      NONE 
; 
; MODIFICATION HISTORY: 
;      - Written by Eric Volquardsen, NASA IRTF, December 2003
;      - Corrected problem with ORIGINAL header keyword when
;        performing multiple registrations, ELV, NASA IRTF, Jan. 2004
;      - Added LIMS keyword, ELV, NASA IRTF, August 2004
; 
; =========================================================================
PRO mirsi_register, infiles, outfile, path, dither = dither, plot = plot, $ 
                    mshft = mshft, step = step, peaks = peaks, lims = lims, $ 
                    help = help

; CHECK INPUTS 
     IF n_elements(help) EQ 1 THEN BEGIN
          IF help NE 0 THEN help = 1 
     ENDIF ELSE help = 0 
     IF n_elements(step) EQ 1 THEN BEGIN
          CASE step OF 
               2 : IF n_elements(infiles) NE 1 THEN help = 1
               1 : IF n_elements(infiles) LT 1 THEN help = 1
               ELSE : BEGIN 
                    step = 0
                    IF n_elements(infiles) LT 2 THEN help = 1
               END
          ENDCASE
     ENDIF ELSE step = 0 
     IF n_elements(outfile) NE 1 THEN help = 1 
     IF help EQ 1 THEN BEGIN
          print, '************************************************'
          print, '***          MIRSI_REGISTER SYNTAX           ***' 
          print, '*** mirsi_register, infiles, outfile, path,  ***'
          print, '***                 dither=dither,           ***'
          print, '***                 plot=plot, mshft=mshft,  ***' 
          print, '***                 step=step, peaks=peaks,  ***'
          print, '***                 lims=lims                ***'
          print, '***                                          ***'
          print, '*** infiles - Names of files to register     ***' 
          print, '***           If step=2 then there should    ***' 
          print, '***           be one name here, otherwise    ***'
          print, '***           more than one.                 ***'
          print, '*** outfile - Filename where registered      ***' 
          print, '***           image will be saved            ***'  
          print, '*** path- 2-element vector listing paths     ***'  
          print, '***       for infiles and outfile            ***'
          print, '*** dither - 2n element vector with x and y  ***' 
          print, '***          displacements for an n-point    ***' 
          print, '***          dither pattern (in pixels)      ***' 
          print, '*** plot - Set to display results            ***'   
          print, '*** mshft - Maximum shift and maximum diff   ***'
          print, '***         between calculated and estimated ***' 
          print, '***         peak positions                   ***'
          print, '*** step - Set for desired registration step ***' 
          print, '***        to execute:                       ***'
          print, '***           0 : Both steps                 ***'
          print, '***           1 : First registration step    ***'
          print, '***           2 : Second registration step   ***'
          print, '*** peaks - Locations of peaks in images     ***' 
          print, '*** lims - Min and max image values          ***' 
          print, '************************************************' 
          RETURN 
     ENDIF  
     np = n_elements(path)
     CASE np OF  
          1 : path = [path, path]
          0 : BEGIN
               v = !version.os_family
               CASE v OF
                    'vms' : root = '[]'
                    'windows' : root = 'c:\'
                    ELSE : root = '/'
               ENDCASE
               cd, root, current=nowp
               cd, nowp
               path = [nowp, nowp]
          END
          ELSE : path = path[0:1]
     ENDCASE
     n = n_elements(infiles)
     IF n_elements(dither) LT 2 THEN dither = -1
     IF n_elements(plot) EQ 1 THEN BEGIN
          IF plot NE 0 THEN plot = 1 
     ENDIF ELSE plot = 0 
     CASE n_elements(mshft) OF 
          0 : mshft = [10, 4]
          1 : mshft = [mshft, 4]
          ELSE : mshft = mshft[0:1]
     ENDCASE 
     IF n_elements(peaks) NE 4*n THEN peaks = -1

; FIRST REGISTRATION
     IF step NE 2 THEN BEGIN 

; LOAD IMAGES
          cofn = ''
          cpos = fltarr(2, n)
          tttt = fltarr(n)
          FOR i = 0, (n-1) DO BEGIN
               GETIM, infiles[i], img, path[0], hdr = hdr
               IF n_elements(img) EQ 1 THEN RETURN
               sz = size(img)
               IF sz[0] NE 3 THEN BEGIN
                    print, infiles[i], ' HAS WRONG IMAGE SIZE'
                    RETURN
               ENDIF 
               IF i EQ 0 THEN images = fltarr(sz[1], sz[2], 2*n)
               images[*, *, (2*i):(2*i+1)] = img  

; Specify original files registered
; (looks for previous registration)
               orig1 = FXPAR(hdr, 'ORIGINAL')
               norig = strpos(orig1, '/')
               IF norig EQ -1 THEN BEGIN 
                    cofn = cofn+' / '+infiles[i]
               ENDIF ELSE BEGIN 
                    IF i EQ 0 THEN cofn = orig1 $ 
                         ELSE cofn = cofn+' / '+orig1
               ENDELSE

; Get observation time
               thist = strtrim(FXPAR(hdr, 'MEANTIME'), 2)
               hr = float(GETWRD(thist, 0, delimiter = ':'))
               minit = float(GETWRD(thist, 1, delimiter = ':'))
               secnd = float(GETWRD(thist, 2, delimiter = ':'))
               tttt[i] = 3600*hr+60*minit+secnd  
          ENDFOR

; PERFORM DITHERING SHIFT (if needed)
          IF dither[0] NE -1 THEN BEGIN
               j = 0 
               FOR i = 0, (n-1) DO BEGIN
                    IF i GT (n_elements(dither)/2-1) THEN i = 0 
                    d_shift = dither[(2*i):(2*i+1)]
                    images[*, *, (2*j):(2*j+1)] = shift(images[*, *, $ 
                          (2*j):(2*j+1)], d_shift[0], d_shift[1], 0)
                    j = j + 1 
                    IF j GT (n-1) THEN GOTO, skp1
               ENDFOR
               skp1: 
          ENDIF

; APPLY LIMITS (if set)
          IF n_elements(lims) EQ 2 THEN BEGIN 
               IF lims[0] GT lims[1] THEN lims = [lims[1], lims[0]]
               images = images > lims[0]
               images = images < lims[1]
          ENDIF 

; CALCULATE MEAN OBSERVATION TIME OF REGISTERED IMAGE
          mntime = total(tttt)/float(n)
          mhr = strtrim(fix(mntime/3600), 2)
          mminit = strtrim(fix((mntime-3600.0*mhr)/60), 2) 
          msecnd = strtrim(round(mntime-3600.0*mhr-60*mminit), 2)
          IF strlen(mhr) EQ 1 THEN mhr = '0'+mhr
          IF strlen(mminit) EQ 1 THEN mminit = '0'+mminit
          IF strlen(msecnd) EQ 1 THEN msecnd = '0'+msecnd
          meantm = mhr+':'+mminit+':'+msecnd

; CALCULATE SHIFT AMOUNTS

;          IF n EQ 1 THEN BEGIN 
;               cor = float(img)
;               mcs = [0, 0]
;               GOTO, skpr
;          ENDIF 
          images1 = fltarr(sz[1], sz[2], n)
          images2 = fltarr(sz[1], sz[2], n)
          mcs =  fltarr(2, (n-1))
          jj =  0 
          FOR ii = 0, (n-1) DO BEGIN

               test1 =  images[0:(sz[1]/2-1), *, 2*ii]
               test2 =  images[0:(sz[1]/2-1), *, 2*ii+1]
               test3 =  images[(sz[1]/2):(sz[1]-1), *, 2*ii]
               test4 =  images[(sz[1]/2):(sz[1]-1), *, 2*ii+1]
               ss1 = IM_STATS(test1[*, *])
               ss2 = IM_STATS(test2[*, *])
               ss3 = IM_STATS(test3[*, *])
               ss4 = IM_STATS(test4[*, *])
               peak1 =  (ss1.peak_xy + ss2.peak_xy)/2.0
               peak2 =  (ss3.peak_xy + ss4.peak_xy)/2.0
               peak = (peak1 + peak2)/2.0
               images1[*, *, ii] = images[*, *, 2*ii]
               images2[*, *, ii] = images[*, *, (2*ii+1)]
               IF peaks[0] NE -1 THEN BEGIN
                    peak[0] = (peaks[4*ii]+peaks[4*ii+2])/2.0
                    peak[1] = (peaks[4*ii+1]+peaks[4*ii+3])/2.0
               ENDIF 
               IF jj EQ 0 THEN BEGIN 
                    jj = 1
                    x0 = peak[0]
                    y0 = peak[1]
                    IF x0 EQ 0 THEN jj = 0
                    IF y0 EQ 0 THEN jj = 0
               ENDIF ELSE BEGIN
                    mcs[0, (ii-1)] = peak[0]-x0
                    mcs[1, (ii-1)] = peak[1]-y0
                    IF abs(mcs[0, (ii-1)]) GT mshft[0] THEN mcs[0, (ii-1)] = 0
                    IF abs(mcs[1, (ii-1)]) GT mshft[0] THEN mcs[1, (ii-1)] = 0
               ENDELSE
          ENDFOR

; CREATE OUTPUT
          cor = fltarr(sz[1], sz[2], 2)
          ADDARR, images1, mcs, tcor
          cor[*, *, 0] = tcor
          ADDARR, images2, mcs, tcor
          cor[*, *, 1] = tcor

; Make adjustments to header
;          skpr: 
          SXDELPAR, hdr, 'MEANTIME'
          SXDELPAR, hdr, 'ORIGINAL'
          FXADDPAR, hdr, 'ORIGINAL', cofn
          FXADDPAR, hdr, 'RCOMMENT', 'Registered by mirsi_register'
          FXADDPAR, hdr, 'MEANTIME', meantm
          FXADDPAR, hdr, 'BITPIX', -32

; Plot (if set)
          IF plot EQ 1 THEN BEGIN
               ttl = '1st REGISTRATION RESULTS FOR '+ strtrim(outfile, 2)
               window, 1, title = ttl, xsize = 2*sz[1], ysize = sz[2]
               wset, 1
               tvscl, cor[*, *, 0]
               tvscl, cor[*, *, 1], sz[1], 0
               print, 'First Registration Shifts'
               print, mcs
          ENDIF 

     ENDIF 

; SECOND REGISTRATION
     IF step NE 1 THEN BEGIN
          
; LOAD SAVED 1st REGISTRATION (if needed)
          IF step EQ 2 THEN BEGIN 
               chk = FILEX(infiles[0], path[0])
               IF chk EQ 0 THEN BEGIN 
                    print, infiles, ' DOES NOT EXIST'
                    RETURN
               ENDIF 
               GETIM, infiles, cor, path[0], hdr = hdr
               sz = size(cor)
               IF sz[0] NE 3 THEN BEGIN
                    print, infiles, ' HAS WRONG IMAGE SIZE'
                    RETURN
               ENDIF 
          ENDIF 

; LOCATE PEAKS
          cc1 = fltarr(5, 108)
          cc2 = fltarr(5, 108)

; Look at overlapping subimages for maxima
          FOR i1 = 0, 11 DO BEGIN
               FOR i2 = 0, 8 DO BEGIN
                    fw1 = FULLWID_HALFMAX(cor[i1*25:(i1*25+30), $ 
                          i2*25:(i2*25+30), 0], centroid = c1)
                    fw2 = FULLWID_HALFMAX(cor[i1*25:(i1*25+30), $ 
                          i2*25:(i2*25+30), 1], centroid = c2)
                    cc1[3, (i2*9+i1)] = i1*25
                    cc1[4, (i2*9+i1)] = i2*25
                    cc2[3, (i2*9+i1)] = i1*25
                    cc2[4, (i2*9+i1)] = i2*25
                    fw1 = total(fw1)/2.0
                    fw2 = total(fw2)/2.0
                    c1[0] = c1[0] < 24 > 1
                    c1[1] = c1[1] < 24 > 1
                    c2[0] = c2[0] < 24 > 1
                    c2[1] = c2[1] < 24 > 1

; Select peaks with reasonable fwhm
                    IF fw1 GT 2.0 THEN BEGIN 
                         cc1[0:1, (i2*9+i1)] = c1
                         cc1[2, (i2*9+i1)] = $ 
                           total(cor[(i1*25+c1[0]-1):(i1*25+c1[0]+1), $ 
                                     (i2*25+c1[1]-1):(i2*25+c1[1]+1), 0])
                    ENDIF
                    IF fw2 GT 2.0 THEN BEGIN 
                         cc2[0:1, (i2*9+i1)] = c2
                         cc2[2, (i2*9+i1)] = $ 
                           total(cor[(i1*25+c2[0]-1):(i1*25+c2[0]+1), $ 
                                     (i2*25+c2[1]-1):(i2*25+c2[1]+1), 1])
                    ENDIF
               ENDFOR
          ENDFOR

; Select two peaks with greatest near-pixel values
          sort1 = reverse(sort(cc1[2, *]))
          sort2 = reverse(sort(cc2[2, *]))
          sort1 = sort1[0:1]
          sort2 = sort2[0:1]
          cxy1 = [cc1[3, sort1[0]]+cc1[0, sort1[0]], $ 
                  cc1[4, sort1[0]]+cc1[1, sort1[0]]]
          cxy2 = [cc1[3, sort1[1]]+cc1[0, sort1[1]], $ 
                  cc1[4, sort1[1]]+cc1[1, sort1[1]]]
          cxy3 = [cc2[3, sort2[0]]+cc2[0, sort2[0]], $ 
                  cc2[4, sort2[0]]+cc2[1, sort2[0]]]
          cxy4 = [cc2[3, sort2[1]]+cc2[0, sort2[1]], $ 
                  cc2[4, sort2[1]]+cc2[1, sort2[1]]]

; Used passed approximate peak positions if passed and 
; separation of calculated peak positions is wrong
          IF peaks[0] NE -1 THEN BEGIN 
               dis1 = sqrt((cxy1[0]-cxy2[0])^2 + (cxy1[1]-cxy2[1])^2)
               dis2 = sqrt((cxy3[0]-cxy4[0])^2 + (cxy3[1]-cxy4[1])^2)
               dis = sqrt((peaks[0]-peaks[2])^2 + (peaks[1]-peaks[3])^2)
               IF abs(dis1-dis) GT mshft[1] THEN BEGIN 
                    cxy1[0] = peaks[0]
                    cxy1[1] = peaks[1]
                    cxy2[0] = peaks[2]
                    cxy2[1] = peaks[3]
               ENDIF 
               IF abs(dis2-dis) GT mshft[1] THEN BEGIN 
                    cxy3[0] = peaks[0]
                    cxy3[1] = peaks[3]
                    cxy4[0] = peaks[2]
                    cxy4[1] = peaks[1]
               ENDIF 
          ENDIF 

; Use calculated positions to create subimage
          c1 = fltarr(2)
          cor1 =  fltarr(81, 81, 4)
          cor1[*, *, 0]= extrac(cor[*, *, 0], (cxy1[0]-40), $ 
              (cxy1[1]-40), 81, 81)
          cor1[*, *, 1]= extrac(cor[*, *, 0], (cxy2[0]-40), $ 
              (cxy2[1]-40), 81, 81)
          cor1[*, *, 2]= extrac(cor[*, *, 1], (cxy3[0]-40), $ 
              (cxy3[1]-40), 81, 81)
          cor1[*, *, 3]= extrac(cor[*, *, 1], (cxy4[0]-40), $ 
              (cxy4[1]-40), 81, 81)

; Calculate subimage shifts
          MCSHIFT, cor1, msh

; SHIFT AND COMBINE SUBIMAGES
          ADDARR, cor1, msh, cor

; Change header values
          SXDELPAR, hdr, 'NAXIS3
          FXADDPAR, hdr, 'NAXIS1', 81
          FXADDPAR, hdr, 'NAXIS2', 81
          FXADDPAR, hdr, 'NAXIS', 2
          FXADDPAR, hdr, 'BITPIX', -32

; Plot (if set)
          IF plot EQ 1 THEN BEGIN
               ttl = '2nd REGISTRATION RESULTS FOR '+ strtrim(outfile, 2)
               window, 2, title = ttl, xsize = 81, ysize = 81
               wset, 2
               tvscl, cor
               wait, 2
          ENDIF 

     ENDIF 

; Add data to file
     cd, path[1], current = nowp
     FXWRITE, outfile, hdr, cor
     cd, nowp

END 







