
; NAME:
;       sky_noise
;
; PURPOSE:
;       Estimate the standard deviation of Gaussian noise by finding the
;       most probable value of standard deviations for moving box of pixels
;       (where the histogram of local standard deviations attains maximum).
;       Then the sky level (average background) can be estimated by
;       fitting a Gaussian of width given by the estimated noise st.dev.
;       to the histogram of the image values.
;
; CALLING:
;       sigma = sky_noise( image, sky_Level, BOX_WIDTH= )
;
; INPUTS:
;       image = 2D array of data.
;
; OUTPUTS:
;       sky_Level = the average value of the background in image,
;               optional, computed only if parameter is present.
;
;       Function returns the most probable standard deviation of noise in image.
;
; KEYWORDS:
;
;       BOX_WIDTH = the width of box around each image pixel
;               in which to compute local variances, default = 3.
;
;       SIGMA_SET = if specficied, this value is used as st.dev. of noise
;               and routine skips to determine the sky_Level.
;
;       /PLOT : create windows with graphs of Histogram of Local Deviations
;               and Histogram of Data, and overplot with derived results.
;
; KEYWORD OPTIONAL OUTPUTS:
;
;       LOCAL_VARIANCE = the image of local variances of data image (no sqrt).
;       HISTO_VARIANCE = histogram of SQUARE-ROOT of local variance image
;       VALUE_VARIANCE = values corresponding to local variance histogram
;       IMAGE_HISTOGRAM = histogram of image
;       IMAGE_VALUES = values  corresponding to image histogram
;
; EXTERNAL CALLS:
;       function Local_Variance
;       function histo
;       pro non_Lin_Lsq
;
; SYSTEM VARIABLES:
;       if !DEBUG GT 3 then histogram plots are produced in temp. window.
; COMMON BLOCKS:
;       common gaussian, sigma_par      ;parameter passed to function gaussian.
;       common sky_noise, winp, winp2   ;window for debug plots.
; PROCEDURE:
;       Find where the histogram of local standard deviations attains maximum,
;       and iterate changing binsize until it is satisfactorily representing
;       the most probable st.dev., then home in on more exact value
;       by nonlinear Lsq fit to the histogram of local st.devs.
;       Then estimate the sky level (average background) by fitting (the mean)
;       a Gaussian with sigma = most probable st.dev. to the histogram of image.
;
; HISTORY:
;       Written, Frank Varosi NASA/GSFC 1992.
;       F.V. 1995, added !DEBUG and /PLOT options.
;       F.V. 1997, improved robustness of initial choice of nbin for histogram.
;-

function sky_noise, image, sky_Level, BOX_WIDTH=box_width,  $
                                        LOCAL_VARIANCE=imvar, $
                                        HISTO_VARIANCE=imvh,  $
                                        VALUE_VARIANCE=imvv,  $
                                        IMAGE_HISTOGRAM=imh,  $
                                        IMAGE_VALUES=imv,     $
                                        SIGMA_SET=sigma_set, PLOT=hplots

  common gaussian, sigma_par    ;parameter passed to function gaussian.
  common sky_noise, winp, winp2

        if keyword_set( sigma_set ) then begin
                sigma_noise = sigma_set
                goto,SKY
           endif

        if N_elements( box_width ) NE 1 then box_width=3
        nbin = 2000
        if( (max( image, MIN=m ) - m)/stdev( image )) GT 60 then nbin = 7000

        imvar = Local_Variance( image, BOX_WID=box_width )
        imsdev = sqrt( imvar( where( imvar GT 0 ) ) )
        imvh = histo( imsdev, imvv, NBIN=nbin )
        imvh(0)=0
        maxh = max( imvh, imax )
        imok = 9
        npf = 0.01
        npix = N_elements( image )
;        if !DEBUG GT 3 then hplots = 1

        if keyword_set( hplots ) then begin
                winsav = !D.window
                get_window, winp, XS=500,YS=400,/SHOW
                print,0,imax,imvv(imax),maxh,nbin,1
           endif

        if (maxh LT npix*npf) OR (imax LT imok) then begin

                npix = N_elements( imsdev )
                nbinp = 0
                iter=0

                while ( (maxh LT npix*npf) OR (imax LT imok) ) AND $
                                     (iter LT 20) AND (nbinp NE nbin) do begin
                        iter = iter+1
                        bfac = 1 + 3/aLog( nbin>2 )
                        nbinp = nbin
                        if (imax LT imok) then nbin = bfac*nbin $
                        else if (maxh LT npix*npf) then nbin = nbin/bfac
                        nbin = Long( nbin < 200000 ) > 20
                        imvh = histo( imsdev, imvv, NBIN=nbin )
                        imvh(0)=0
                        maxh = max( imvh, imax )

                        if keyword_set( hplots ) then begin
                                np = ( 3*imax ) < (N_elements( imvv )-1)
                                plot,imvv(0:np),imvh(0:np),PSYM=10
                                print,iter,imax,imvv(imax),maxh,nbin,bfac
                           endif
                  endwhile
           endif

        if keyword_set( hplots ) then begin
                np = ( 3*imax ) < (N_elements( imvv )-1)
                plot,imvv(0:np),imvh(0:np),PSYM=10,XTIT="Stand. Deviation", $
                        YTIT="Frequency", TIT="Histogram of Local Deviations"
                help,imvv,imvh
                print," mode =",imvv(imax),"    maxh =",maxh,"    at",imax
;                if !DEBUG GT 4 then stop
           endif

        if (imax gt 1) then begin

           pg = [ maxh, imvv(imax), (imvv(imax)-imvv(0))/2/sqrt(2*aLog(2)) ]
           np = ( 3*imax ) < (N_elements( imvv )-1)   ;fit to variance histogram

                non_Lin_Lsq, imvv(0:np), imvh(0:np), pg, pgsig, FUNC="gaussian"

                if (pg(1) GT 0) then begin
                        sigma_noise = pg(1)
                        if keyword_set( hplots ) then $
                                oplot, imvv(0:np), gaussian( imvv(0:np), pg )
                 endif else sigma_noise = imvv(imax)

                if keyword_set( hplots ) then begin
                        oplot, replicate( sigma_noise, 2 ), !y.crange
                        if (winsav GE 0) then wset,winsav
                   endif

         endif else begin

                sigma_noise = 0
                sky_Level = 0
                return, sigma_noise
                if keyword_set( hplots ) then begin
                        if (winsav GE 0) then wset,winsav
                   endif
          endelse

        if N_params() LT 2 then return, sigma_noise
SKY:
        sigma_par = sigma_noise     ;fix sigma parameter of gaussian function.

        imh = histo( image, imv, BINSIZE=(sigma_par/7), NBIN=10000 )

        imh(0)=0                                ;ignore background pixels
        pg = [ max( imh, imax ), imv(imax) ]    ;guess factor and mean,
        np = ( 2*imax ) < (N_elements( imv )-1) ; fit to histogram around Max.

        non_Lin_Lsq, imv(0:np), imh(0:np), pg, pgsig, FUNC="gaussian"

        sky_Level = pg(1)       ;estimate of gaussian mean (sky background).

        if keyword_set( hplots ) then begin
                winsav = !D.window
                get_window, winp2, XS=500,YS=400
                plot, imv(0:np), imh(0:np), XTIT="Value", YTIT="Frequency", $
                                                TIT="Histogram of Data", PS=10
                oplot, imv(0:np), gaussian( imv(0:np), pg )
                oplot, replicate( sky_Level, 2 ), !y.crange
                if (winsav GE 0) then wset,winsav
           endif

return, sigma_noise
end
